// groovy.groovy

// Groovy supports Jython-style lists. Because of groovy's dynamic typing, you
// do not need to specify a variable's type. You also don't need to specify
// semicolon terminators.

months = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep",
          "Oct", "Nov", "Dec"]

// You can use a Jython-style for loop to output the list's items.

for (month in months)
   println month

// You can use an inclusive range if you want to access list items by index.

for (monthIndex in 0..11)
   println months[monthIndex]

// Classes can include properties.

class Salesman
{
   String name // name is a property of Salesman
}

class Author
{
   String name // name is a property of Author
}

// Object creation is similar to Java.

s = new Salesman()

// Properties can be accessed directly.

s.name = "John Doe"

a = new Author()

// Properties can be accessed via getter/setter methods -- with or without
// surrounding parentheses.

a.setName "Jane Smith"

people = [s, a]
for (i in people)
{
   println i.name
   i.setName (i.name.reverse()) // name must be of String type
   outputPerson (i)
}

// Functions can be defined globally outside of classes. For this function,
// any object with a name property can be passed as an argument.

def outputPerson(person)
{                       
   println person.getName ()
}

// The following class demonstrates the usefulness of closures.

class Formatter
{
   Object format

   def doFormat (p, Object[] args)
   {
      format.call (p, args) // Invoke closure with initial pattern and a...
                            // ...variable number of arguments.
   }
}

// Imports can be placed where needed.

import java.text.*

// Create a currency formatter closure.

curFormatter =
{
   value, Object[] args ->
     println NumberFormat.getCurrencyInstance ().format (value)
}

// Create a message formatter closure.

msgFormatter =
{
   pattern, Object[] args ->
     println MessageFormat.format (pattern, args)
}

// Create a Formatter with a message formatter closure.

f = new Formatter (format: msgFormatter)

// Format a generic message.

f.doFormat ("The date is {0, date}; the time is {0, time}",
            new java.util.Date())

// Create a Formatter with a closure for strictly formatting currency values.

f = new Formatter (format: curFormatter)

// Format a currency value.

f.doFormat (137.9256)
